ChangeTokenSource

Trait ChangeTokenSource 

pub trait ChangeTokenSource<T: Value> {
    // Required method
    fn token(&self) -> Box<dyn ChangeToken>;

    // Provided method
    fn name(&self) -> &str { ... }
}
Used to fetch a change token to track options changes.

Required Methods

fn token(&self) -> Box<dyn ChangeToken>

Creates and returns a change token which can be used to register a change notification callback.

Provided Methods

fn name(&self) -> &str

Gets the name of the option instance being changed, if any.

Implementors