pub trait ConfigExt<C>: Sized {
// Required methods
fn apply_config<T>(&mut self, configuration: C) -> Builder<'_, T>
where T: Value + Default + DeserializeOwned + Send + Sync + 'static;
fn apply_config_at<T>(
&mut self,
configuration: C,
key: impl AsRef<str>,
) -> Builder<'_, T>
where T: Value + Default + DeserializeOwned + Send + Sync + 'static;
}Available on crate feature
cfg only.Expand description
Defines configuration extension methods for a ServiceCollection.
Required Methods
Sourcefn apply_config<T>(&mut self, configuration: C) -> Builder<'_, T>
fn apply_config<T>(&mut self, configuration: C) -> Builder<'_, T>
Registers an options type that will have all of its associated services registered.
Arguments
configuration- The configuration applied to the options
Sourcefn apply_config_at<T>(
&mut self,
configuration: C,
key: impl AsRef<str>,
) -> Builder<'_, T>
fn apply_config_at<T>( &mut self, configuration: C, key: impl AsRef<str>, ) -> Builder<'_, T>
Registers an options type that will have all of its associated services registered.
Arguments
configuration- The configuration applied to the optionskey- The key to the part of the configuration applied to the options
Dyn Compatibility
This trait is not dyn compatible.
In older versions of Rust, dyn compatibility was called "object safety", so this trait is not object safe.
Implementations on Foreign Types
Sourceimpl<C> ConfigExt<C> for ServiceCollection
Available on crate features di or cfg only.
impl<C> ConfigExt<C> for ServiceCollection
Available on crate features
di or cfg only.