pub trait ChangeTokenSource<T: Value> {
// Required method
fn token(&self) -> Box<dyn ChangeToken>;
// Provided method
fn name(&self) -> &str { ... }
}Expand description
Used to fetch a change token to track options changes.
Required Methods
Sourcefn token(&self) -> Box<dyn ChangeToken>
fn token(&self) -> Box<dyn ChangeToken>
Creates and returns a change token which can be used to register a change notification callback.