pub trait Snapshot<T: Value> {
// Required method
fn get_named(&self, name: &str) -> Result<Ref<T>, Error>;
// Provided methods
fn get(&self) -> Result<Ref<T>, Error> { ... }
fn get_unchecked(&self) -> Ref<T> { ... }
fn get_named_unchecked(&self, name: &str) -> Ref<T> { ... }
}Expand description
Defines the behavior for a snapshot of configuration options.
Required Methods
Provided Methods
Sourcefn get_unchecked(&self) -> Ref<T>
fn get_unchecked(&self) -> Ref<T>
Gets the default, unnamed configuration options.
Remarks
This function panics if the configuration options could not be successfully retrieved.